@@ -105,7 +105,8 @@ find_neighbour(const unsigned char *address, struct network *net)
     return neigh;
 }
 
-/* Recompute a neighbour's rxcost.  Return true if anything changed. */
+/* Recompute a neighbour's rxcost.  Return true if anything changed.
+   This does not call local_notify_neighbour, see update_neighbour_metric. */
 int
 update_neighbour(struct neighbour *neigh, int hello, int hello_interval)
 {
@@ -121,7 +122,7 @@ update_neighbour(struct neighbour *neigh, int hello, int hello_interval)
             (neigh->hello_interval * 10);
         if(missed_hellos <= 0)
             return rc;
-        timeval_plus_msec(&neigh->hello_time, &neigh->hello_time,
+        timeval_add_msec(&neigh->hello_time, &neigh->hello_time,
                           missed_hellos * neigh->hello_interval * 10);
     } else {
         if(neigh->hello_seqno >= 0 && neigh->reach > 0) {
@@ -190,8 +191,6 @@ update_neighbour(struct neighbour *neigh, int hello, int hello_interval)
         send_unicast_request(neigh, NULL, 0);
         send_ihu(neigh, NULL);
     }
-    if(rc)
-        local_notify_neighbour(neigh, LOCAL_CHANGE);
     return rc;
 }
 
@@ -248,10 +247,7 @@ check_neighbours()
         rc = reset_txcost(neigh);
         changed = changed || rc;
 
-        if(changed) {
-            update_neighbour_metric(neigh);
-            local_notify_neighbour(neigh, LOCAL_CHANGE);
-        }
+        update_neighbour_metric(neigh, changed);
 
         if(neigh->hello_interval > 0)
             msecs = MIN(msecs, neigh->hello_interval * 10);
