@@ -35,6 +35,8 @@ struct network_conf {
     char wired;
     char split_horizon;
     char lq;
+    char faraway;
+    int channel;
     struct network_conf *next;
 };
 
@@ -46,6 +48,12 @@ struct network_conf {
 #define NET_WIRED (1<<1)
 #define NET_SPLIT_HORIZON (1 << 2)
 #define NET_LQ (1 << 3)
+#define NET_FARAWAY (1 << 4)
+
+/* Only INTERFERING can appear on the wire. */
+#define NET_CHANNEL_UNKNOWN 0
+#define NET_CHANNEL_INTERFERING 255
+#define NET_CHANNEL_NONINTERFERING -2
 
 struct network {
     struct network *next;
@@ -53,6 +61,7 @@ struct network {
     unsigned int ifindex;
     unsigned short flags;
     unsigned short cost;
+    int channel;
     struct timeval hello_timeout;
     struct timeval update_timeout;
     struct timeval flush_timeout;
